@@ -8,6 +8,9 @@ import sys
 from datetime import datetime
 import uuid
 from python_cassandra_cli.aws_utils import AWSUtils
+from cassandra import ReadTimeout
+from cassandra.concurrent import execute_concurrent
+import time
 
 client_s3 = boto3.client('s3')
 
@@ -18,7 +21,7 @@ dotenv.load_dotenv()
 class RestoreBackupKeyspace(object):
 
     def __init__(self, s3, id, key, keyspace, region, schema, snapshot_folder, host, user, password,
-        ssm_secret, secret_name, drop_keyspace, rows_limit):
+        ssm_secret, secret_name, drop_keyspace, data_insert_chunk_size, csv_field_size_limit, session_concurrency_level):
 
         self.s3 = s3
         self.id = id
@@ -33,7 +36,9 @@ class RestoreBackupKeyspace(object):
         self.ssm_secret = ssm_secret
         self.secret_name = secret_name
         self.drop_keyspace = drop_keyspace
-        self.rows_limit = rows_limit
+        self.data_insert_chunk_size = data_insert_chunk_size
+        self.csv_field_size_limit = csv_field_size_limit
+        self.session_concurrency_level = session_concurrency_level
 
     def restore_backup_keyspace(self) :
 
@@ -67,10 +72,12 @@ class RestoreBackupKeyspace(object):
                 user=result_dict['USER']
                 password=result_dict['PASSWORD']
             else:
-                user = str(os.getenv('CASSANDRA_HOST_USER')) or self.user
-                password = str(os.getenv('CASSANDRA_HOST_PASSWORD')) or self.password
+                user = self.user
+                password = self.password
 
-            host = os.getenv('CASSANDRA_HOST') or self.host
+            host = self.host
+
+            concurrency_level =  self.session_concurrency_level
 
             # connecting to cassandra
             auth_provider = PlainTextAuthProvider(username=user, password=password)
@@ -156,14 +163,19 @@ class RestoreBackupKeyspace(object):
                     values_str = ','.join(values_dict)
 
                     insert_data_stmt = session.prepare(f"INSERT INTO {self.keyspace}.{table_name} ({table_meta_str}) VALUES ({values_str});")
+
+                    chunk_size = self.data_insert_chunk_size
+                    chunk_count = 0
                     #increase the limit for csv
-                    csv.field_size_limit(os.getenv('CSV_FIELD_SIZE_LIMIT') or 100000000)
+                    csv.field_size_limit(self.csv_field_size_limit)
                     with open(self.snapshot_folder + '/' + table_name + '.csv', newline='') as csvfile:
+                        start = time.time()
                         data = csv.reader(csvfile, delimiter='|')
+                        rows_count = 0
+                        statements_and_params = []
                         for row in data:
-                            # print(row)
+                            rows_count += 1
                             for i in range(len(row)):
-                                print(data_types_list[i])
                                 if row[i]== 'null':
                                     row[i] = None
                                 elif data_types_list[i] == 'timestamp':
@@ -196,15 +208,21 @@ class RestoreBackupKeyspace(object):
                                         row[i] = row[i][1:]
                                     row[i] =row[i].replace("'","").split(",")
                                     row[i] = set([item.strip() for item in row[i]])
+                            
+                            params = (row)
+                            statements_and_params.append((insert_data_stmt, params))
+                            if rows_count % chunk_size == 0:
+                                 chunk_count += 1
+                                 print(f'Inserting {chunk_count} chunk of data')
+                                 execute_concurrent(session, statements_and_params,concurrency=concurrency_level , raise_on_first_error=False)
+                                 statements_and_params = []
+                        if len(statements_and_params) > 0:
+                            print("Inserting rest of data")
+                            execute_concurrent(session, statements_and_params,concurrency=concurrency_level , raise_on_first_error=False)
+                        end = time.time()
+                        print(f"Data into table {table_name} uploaded: ",(end-start),"sec")
 
-                            session.execute(insert_data_stmt, row)
-
-                    rows_count = session.execute(f"SELECT * from {self.keyspace}.{table_name} LIMIT {self.rows_limit}")
-
-                    data_count_current = 0
-                    for row in rows_count:
-                        data_count_current += 1
-                   
+                    data_count_current = rows_count                  
 
                     print(f"Data count in table {table_name} origin equal {data_count_origin}")
                     print(f"Data count in table {table_name} current equal {data_count_current}")
